/****************************************************************************
*
*   Module Name: PASSWORD
*
*   Description:
*       This is the application's main module.  This module starts the
*   application, dispatches events, and stops the application.
*
*   Edit History:
*       2/15/96     Created
*
****************************************************************************/

#include "main.h"


/**************************************************************
*                           M A I N
**************************************************************/

int PASCAL WinMain(
	HANDLE hInstance,
	HANDLE hPrevInstance,
	LPSTR lpszCmdParam,
	int nCmdShow
)
{
	WORD error = eMsOK;
	WORD rscFileId;  // handle of application's resource file

	rscFileId = StartApplication(
		hInstance,
		hPrevInstance,
		lpszCmdParam,
		nCmdShow,
		&error
	);

	if (!CheckError(error))
	{
		EventLoop();
	}

	// release all resources and exit the application
	StopApplication(rscFileId);

	return 0;
}

/************************************************************
*
*                      StartApplication
*
*       This routine initializes the PenRight! API, opens
*   the application's resource file, and posts an application
*   enter event.  The resources of an application are
*   typically appended to the end of the application's .exe
*   file.  However, for debugging purposes you need to have
*   the resources in a separate file.  If this routine does
*   not find the resources in the .exe file, it will look
*   for them in a file with the same file name as the .exe
*   file but with an extension of ".rsc" in the same
*   directory as the .exe file.
*
************************************************************/

WORD StartApplication(
	HANDLE hInstance,
	HANDLE hPrevInstance,
	LPSTR lpszCmdParam,
	int nCmdShow,
	WORD *pError
)
{
	WORD initError, rscFileId;
	char far *path;
	char msg[100];
	static char exePath[255];


	initError = ApiInitialize(hInstance, hPrevInstance, lpszCmdParam, nCmdShow);
	if (initError)
	{
		sprintf(msg, "PenRight! initialization error: %u\n", initError);
		MessageBox(0, msg, 0, MB_OK | MB_ICONEXCLAMATION | MB_TASKMODAL);
		DestroyWindow(ApiGetWindowHandle());
		exit(initError - 9000);
	}

	GetModuleFileName(hInstance, exePath, sizeof(exePath));

	rscFileId = RscOpenResourceFile(exePath, pError);
	if (*pError)
	{
		path = strdup(exePath);
		if (path)
		{
			strcpy(&path[strlen(path) - 3], "Rsc");
			rscFileId = RscOpenResourceFile(path, pError);
			free(path);
		}
	}

	PostEvent(appEnterEvent, 0);
	return(rscFileId);
}

/*************************************************************
*                        EventLoop
*************************************************************/

void EventLoop(void)
{
	WORD error;
	EVENTTYPE event;
	BOOLEAN handled;

	do
	{
		EvtGetEvent(&event, &error);
		if (CheckError(error)) break;

		handled = CurrentFormHandleEvent(&event, &error);
		if (CheckError(error)) break;

		if (! handled)
			handled = ApplicationHandleEvent(&event, &error);
		if (CheckError(error)) break;

		if (! handled)
			handled = CodeStartHandleEvent(&event, &error);
		if (CheckError (error)) break;

		if (! handled)
			handled = FrmHandleEvent (&event, &error);
		if (CheckError (error)) break;
	}
	while (event.eType != appExitEvent);
}

/*************************************************************
*                     CodeStartHandleEvent
*************************************************************/

BOOLEAN CodeStartHandleEvent(EVENTTYPE *pEvent, WORD *pError)
{
	BOOLEAN handled = FALSE;

	*pError = eMsOK;
	switch (pEvent->eType)
	{
		case frmInitEvent:
			InitForm(pEvent->data1);
			*pError = GetErrorCode();
			break;

		case frmOpenEvent:
			OpenForm(pEvent->data1);
			*pError = GetErrorCode();
			break;

		case frmCloseEvent:
			CloseForm(pEvent->data1);
			break;
	}
	return handled;
}

/*************************************************************
*
*                      StopApplication
*
*  This routine terminates the PenRight! API and closes
*  the application's resource file.
*
*************************************************************/

void StopApplication(WORD rscFileId)
{
	WORD error;

	RscCloseResourceFile(rscFileId, &error);
	DestroyWindow(ApiGetWindowHandle());

	exit(0);
}

/*************************************************************
*
*                       CheckError
*
*       This routine checks the error code it is passed.  If
*   the error code is not zero, an error message will be
*   displayed in a dialog box.  The dialog box will contain
*   two buttons: one to indicate that processing should
*   continue, and one to indicate that processing should be
*   terminated.  This routine will return TRUE if processing
*   should be terminated, or FALSE if processing should
*   continue.
*
*************************************************************/

BOOLEAN CheckError(WORD error)
{
	WORD ctlPressed, localError;
	char msg[100];
	char *ctl1  = "Continue";
	char *ctl2  = "Exit";
	char *title = "Warning";

	if (error == 0)
		return FALSE;
	else
	{
		if (error <= sys_nerr)
			sprintf(msg, "Error %d: %.80s", error, strerror(error));
		else
			sprintf(msg, "PenRight! Error %d", error);
		ctlPressed = WinDisplayNotice(
			NULL, msg, strlen(msg),
			ctl1, strlen(ctl1),
			ctl2, strlen(ctl2),
			title,
			&localError
		);
		if (!localError)
			return (ctlPressed == 2);
		MessageBox(0, msg, 0, MB_OK | MB_ICONEXCLAMATION | MB_TASKMODAL);
		return TRUE;
	}
}

/**************************************************************
*
*                        GetObjectId
*
*       This routine returns the id of the object (field,
*   list, or control) in which the event occured.  If the
*   event did not occur in a field, list, or control then
*   the return value will be zero.
*
**************************************************************/

WORD GetObjectId(EVENTTYPE *pEvent)
{
	WORD curFrm, curFld;

	switch (pEvent->eType)
	{
		case ctlEnterEvent:
		case ctlExitEvent:
		case lstEnterEvent:
		case lstChangedEvent:
		case lstExitEvent:
		case fldEnterEvent:
		case fldChangedEvent:
		case ctlPressedEvent:
			return pEvent->data1;

		case keyDownEvent:
			curFrm = WinGetActiveWindow();
			curFld = FrmGetCurrentField(curFrm);
			if (curFld)
				return FrmGetFieldId(curFrm, curFld);
			else
				return 0;

		default:
			return 0;
	}
}

